package jboss.arquillian.test1;

import java.io.InputStream;

import javax.inject.Inject;

import junit.framework.Assert;

import org.jboss.arquillian.container.test.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.osgi.testing.OSGiManifestBuilder;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.asset.Asset;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.junit.Test;
import org.junit.runner.RunWith;
import org.osgi.framework.Bundle;
import org.osgi.framework.BundleContext;
import org.osgi.framework.ServiceReference;

@RunWith(Arquillian.class)
public class SimpleArquillianTestCase {

	@Inject
	public Bundle bundle;

	@Deployment
	public static JavaArchive createdeployment() {

		final JavaArchive archive = ShrinkWrap.create(JavaArchive.class,
				"example-arquillian.jar");

		archive.addPackage(Activator.class.getPackage());

		archive.setManifest(new Asset() {
			public InputStream openStream() {
				OSGiManifestBuilder builder = OSGiManifestBuilder.newInstance();

				builder.addBundleSymbolicName(archive.getName());

				builder.addBundleManifestVersion(2);

				builder.addBundleActivator(Activator.class.getName());
				builder.addExportPackages(SimpleArquillianTestCase.class);

				builder.addDynamicImportPackages("*");

				return builder.openStream();

			}

		});

		return archive;

	}

	@Test
	public void testBundleInjection() throws Exception {
		// Assert that the bundle is injected

		Assert.assertNotNull("Bundle injected", bundle);

		// Assert that the bundle is in state RESOLVED

		// Note when the test bundle contains the test case it

		// must be resolved already when this test method is called

		Assert.assertEquals("Bundle RESOLVED", Bundle.RESOLVED,
				bundle.getState());

		// Start the bundle

		bundle.start();

		Assert.assertEquals("Bundle ACTIVE", Bundle.ACTIVE, bundle.getState());

		// Get the service reference

		BundleContext context = bundle.getBundleContext();

		ServiceReference sref = context.getServiceReference(Service.class
				.getName());

		Assert.assertNotNull("ServiceReference not null", sref);

		// Get the service for the reference

		Service service = (Service) context.getService(sref);

		Assert.assertNotNull("Service not null", service);

		// Invoke the service

		int sum = service.sum(1, 2, 3);

		Assert.assertEquals(6, sum);

		// Stop the bundle

		bundle.stop();

		Assert.assertEquals("Bundle RESOLVED", Bundle.RESOLVED,
				bundle.getState());

	}

}